package controllers;

import java.io.IOException;
import java.sql.Timestamp;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmLibraryDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.FilmLibrary;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/filmlibrary")
public class FilmLibraryController {

	private @Autowired FilmLibraryDAO filmLibraryDAO;
	private @Autowired UserDAO userDAO;
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/add", method=RequestMethod.POST)
	@ResponseBody
	public Timestamp addToFilmlibrary(int filmSid, boolean gotIt, Boolean seeWannaSee, String hash) throws UnauthorizedException, IOException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					filmLibraryDAO.add(filmSid, userSid, gotIt, seeWannaSee);
					return filmLibraryDAO.get(filmSid, userSid).getRecordDate();
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/update", method=RequestMethod.POST)
	@ResponseBody
	public void updateFilmLibrary(int filmSid, boolean gotIt, Boolean seeWannaSee, String hash) throws UnauthorizedException, IOException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					filmLibraryDAO.update(filmSid, userSid, gotIt, seeWannaSee);
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/get", method=RequestMethod.GET)
	@ResponseBody
	public FilmLibrary getFilmLibrary(int filmSid, String hash) throws UnauthorizedException, IOException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					return filmLibraryDAO.get(filmSid, userSid);
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
	
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
}
