package services;

import java.util.List;

import org.springframework.dao.DataAccessException;

import dto.Member;

public interface MemberDAO {
	
	/**
	 * Creates a new member
	 * @param meetingSid int
	 * @param userSid String
	 * @param gotIt	boolean
	 * @throws DataAccessException
	 */
	void create(int meetingSid, String userSid, boolean gotIt) throws DataAccessException;
	
	/**
	 * Gets a member
	 * @param meetingSid int
	 * @param userSid String
	 * @return Member returned
	 * @throws DataAccessException
	 */
	Member get(int meetingSid, String userSid) throws DataAccessException;
	
	/**
	 * Gets all members for a meeting
	 * @param meetingSid int
	 * @return List<Member> returned
	 * @throws DataAccessException
	 */
	List<Member> getAll(int meetingSid) throws DataAccessException;
	
	/**
	 * Updates voted
	 * @param meetingSid int
	 * @param userSid String
	 * @throws DataAccessException
	 */
	void updateVoted(int meetingSid, String userSid) throws DataAccessException;
	
	/**
	 * Checks if all members have voted
	 * @param meetingSid int
	 * @return boolean returned
	 * @throws DataAccessException
	 */
	boolean checkAllVoted(int meetingSid) throws DataAccessException;
}
