package services;

import java.util.List;

import org.springframework.dao.DataAccessException;

import dto.Notification;

public interface NotificationDAO {
	
	/**
	 * Creates notification of a meeting
	 * @param senderSid String
	 * @param receiverSid String
	 * @param meetingSid int
	 * @param voted boolean
	 * @throws DataAccessException
	 */
	void createMeeting(String senderSid, String receiverSid, int meetingSid, boolean voted) throws DataAccessException;
	
	/**
	 * Creates notification of a new friend
	 * @param senderSid String
	 * @param receiverSid String
	 * @throws DataAccessException
	 */
	void createNewFriend(String senderSid, String receiverSid) throws DataAccessException;
	
	/**
	 * Updates to seen a notification
	 * @param notificationSid int
	 * @throws DataAccessException
	 */
	void seen(int notificationSid) throws DataAccessException;
	
	/**
	 * Gets all notifications
	 * @param userSid String
	 * @return List<Notification> returned
	 * @throws DataAccessException
	 */
	List<Notification> getNotifications(String userSid) throws DataAccessException;
	
	/**
	 * Indicates if there are notifications not seen
	 * @param userSid String
	 * @return List<Notification> returned
	 * @throws DataAccessException
	 */
	boolean news(String userSid)  throws DataAccessException;
}
