package services.impl;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.MeetingDAO;
import dto.Meeting;
 
@Repository
public class MeetingDAOImpl implements MeetingDAO{
	
	private @Autowired GeneralDAO generalDAO;
	
	@Transactional(readOnly = false)
	public int create(final String name, final String userSid, final int numberCandidates, final boolean inCinema, final boolean notSeen, final Timestamp date) throws DataAccessException{
		try{
			final String query = "INSERT INTO meeting (name, user_sid, number_candidates, in_cinema, not_seen, voted, date) VALUES (?, ?, ?, ?, ?, ?, ?)";

			KeyHolder holder = new GeneratedKeyHolder();
			generalDAO.getJdbcTemplate().update(new PreparedStatementCreator() {           

                @Override
                public PreparedStatement createPreparedStatement(Connection connection)
                        throws SQLException {
                    PreparedStatement ps = connection.prepareStatement(query, Statement.RETURN_GENERATED_KEYS);
                    ps.setString(1, name);
                    ps.setString(2, userSid);
                    ps.setInt(3, numberCandidates);
                    ps.setBoolean(4, inCinema);
                    ps.setBoolean(5, notSeen);
                    ps.setBoolean(6, false);
                    ps.setTimestamp(7, date);
                    return ps;
                }
            }, holder);
			
			// Sid returned
			return holder.getKey().intValue();
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public Meeting get(int sid) throws DataAccessException{
		try{
			String query = "SELECT * FROM meeting WHERE sid = ?";
			Object[] params = new Object[]{
				sid	
			};
			Meeting meeting = generalDAO.getJdbcTemplate().queryForObject(query, params, new BeanPropertyRowMapper<Meeting>(Meeting.class));
			return meeting;
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}	
	
	@Transactional(readOnly = true)
	public List<Meeting> getNextMeetings(String userSid) throws DataAccessException{
		try{
			String query = "SELECT mt.* FROM meeting mt INNER JOIN member m ON m.meeting_sid=mt.sid WHERE m.user_sid=? AND mt.date>? AND mt.voted=true ORDER BY mt.date ASC";
			Timestamp time = new Timestamp(System.currentTimeMillis());
			Object[] params = new Object[]{
				userSid, time
			};
			List<Meeting> meetingList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Meeting>(Meeting.class));
			return meetingList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = true)
	public List<Meeting> myMeetings(String userSid) throws DataAccessException {
		try{
			String query = "SELECT mt.* FROM meeting mt INNER JOIN member m ON m.meeting_sid=mt.sid WHERE m.user_sid=? AND mt.date>? ORDER BY mt.date ASC";
			Timestamp time = new Timestamp(System.currentTimeMillis());
			Object[] params = new Object[]{
				userSid, time
			};
			List<Meeting> meetingList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Meeting>(Meeting.class));
			return meetingList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = false)
	public void updateVoted(int sid) throws DataAccessException {
		try{
			String query = "UPDATE meeting SET voted=true, date=date WHERE sid=?";
			Object[] params = new Object[]{
				sid
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}